import logging

import starling.utils.request as request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

make_local_payment_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "destinationAccountUid", "validations": ("required", str)},
    {"name": "reference", "validations": ("required", str)},
    {"name": "amount", "validations": ("required", str)}
]

list_scheduled_payments_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]


class Payment(object):
    """ Service to interact with a customer's payments """

    def __init__(self, options):
        """
        Creates an instance of the payment service

        :param options: application config
        """
        self.options = options

    def make_local_payment(self, access_token, destination_account_uid, reference, amount):
        """
        Makes a payment on behalf of the customer to another UK bank account using the Faster Payments network

        :param access_token: the oauth bearer token
        :param destination_account_uid:
        :param reference: the payment reference, max. 18 characters
        :param amount: the amount to be send
        :return: the json response dict
        """
        type_validation([access_token, destination_account_uid, reference, amount],
                        make_local_payment_parameter_definition)
        url = "{api_url}/api/v1/payments/local".format(api_url=self.options["api_url"])
        logging.debug("POST {url}".format(url=url))
        return request.post(url, headers=default_headers(access_token), data={
            "destinationAccountUid": destination_account_uid,
            "payment": {
                "amount": amount,
            },
            "reference": reference
        })

    def list_scheduled_payments(self, access_token):
        """
        Lists the customer's scheduled payments

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], list_scheduled_payments_parameter_definition)
        url = "{api_url}/api/v1/payments/scheduled".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))



"""
The module that represents any HTTP request related logic.
Delegates to the requests library with Starling-specific validation.

Adapted from https://github.com/muyiwaolu/monzo-python/blob/master/monzo/request.py
"""
from json import JSONDecodeError

import requests

from starling.errors import (BadRequestError, UnauthorizedError, ForbiddenError,
                             InternalServerError, NotFoundError, UnhandledStatusCodeError)


def get(url, headers=None, params=None):
    """Sends a GET request to a specified URL
       :param url: The URL to send the request to
       :param headers: The headers to include with the request
       :param params: The data to include with the request
       :rtype: A Dictionary representation of the response
    """
    response = requests.get(url=url, headers=headers, params=params)
    return _validate_response(response)


def post(url, headers=None, params=None, data=None):
    """Sends a POST request to a specified URL
       :param url: The URL to send the request to
       :param headers: The headers to include with the request
       :param params: The data to include with the request
       :rtype: A Dictionary representation of the response
    """
    response = requests.post(url=url, headers=headers, data=data, params=params)
    return _validate_response(response)


def put(url, headers=None, params=None, data=None):
    """Sends a PUT request to a specified URL
       :param url: The URL to send the request to
       :param headers: The headers to include with the request
       :param params: The data to include with the request
       :rtype: A Dictionary representation of the response
    """
    response = requests.put(url=url, headers=headers, data=data, params=params)
    return _validate_response(response)


def delete(url, headers=None, params=None, data=None):
    """Sends a DELETE request to a specified URL
       :param url: The URL to send the request to
       :param headers: The headers to include with the request
       :param params: The data to include with the request
       :rtype: A Dictionary representation of the response
    """
    response = requests.delete(url=url, headers=headers, data=data, params=params)
    return _validate_response(response)


def _validate_response(response):
    """Validate the response and raises any appropriate errors.
       :param response: The response to validate
       :rtype: A Dictionary representation of the response, if no errors occured.
    """

    if response.status_code == 204:
        return {}

    try:
        json_response = response.json()
    except JSONDecodeError:
        json_response = {}

    if response.status_code == 200 or response.status_code == 202:
        return json_response
    if response.status_code == 400:
        raise BadRequestError(json_response["error_description"])
    if response.status_code == 401:
        raise UnauthorizedError(json_response["error_description"])
    if response.status_code == 403:
        raise ForbiddenError(json_response["error_description"])
    if response.status_code == 404:
        raise NotFoundError(json_response["error_description"])
    if response.status_code == 400:
        raise InternalServerError(json_response["error_description"])

    raise UnhandledStatusCodeError(response.status_code)



import logging

import starling.utils.request as request
from starling.errors import ClientConfigurationError
from starling.utils.http import oauth_headers
from starling.utils.validator import type_validation

refresh_token_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

authorization_code_parameter_definition = [
    {"name": "authorizationCode", "validations": ("required", str)}
]

ACCESS_TOKEN_GRANT_TYPE = "authorization_code"
REFRESH_TOKEN_GRANT_TYPE = "refresh_token"


class OAuth(object):
    """Service to interact with the oauth endpoint"""

    def __init__(self, options):
        """
        Creates a instance of the oauth service

        :param options: application config
        """
        self.options = options

    def get_access_token(self, authorization_code):
        """
        Exchanges the authorization code for an access token

        :param authorization_code: the authorization code, acquired from the user agent after the
                                   user authenticates with starling
        :return: the json response dict
        """
        type_validation([authorization_code], authorization_code_parameter_definition)

        return self.get_oauth_token({
            "code": authorization_code,
            "grant_type": ACCESS_TOKEN_GRANT_TYPE,
            "client_id": self.options["client_id"],
            "client_secret": self.options["client_secret"],
            "redirect_uri": self.options["redirect_uri"]
        })

    def refresh_access_token(self, refresh_token):
        """
        Exchanges the authorization code for an access token

        :param refresh_token: the oauth refresh token, used when the access token
                              expires to claim a new access token.
        :return: the json response dict
        """
        type_validation([refresh_token], refresh_token_parameter_definition)

        return self.get_oauth_token({
            "refresh_token": refresh_token,
            "grant_type": REFRESH_TOKEN_GRANT_TYPE,
            "client_id": self.options["client_id"],
            "client_secret": self.options["client_secret"],
        })

    def get_oauth_token(self, params):
        """
        Gets the access token from the starling oauth endpoint

        :param params: the query params passed to the oauth endpoint as per the oauth spec
        :return: the json response dict
        """
        if not self.options["client_id"]:
            raise ClientConfigurationError("clientId is not configured")

        if not self.options["client_secret"]:
            raise ClientConfigurationError("clientSecret is not configured")

        url = "{url}/oauth/access-token".format(url=self.options["oauth_url"])
        logging.debug("POST {url}".format(url=url))
        return request.post(url, headers=oauth_headers(), params=params)



import json

from starling.starling import Starling
from starling.utils.http import default_headers
from test.testsupport import expect_authorization_header


def test_starling_client(requests_mock):
    """test static access token provided in client instance"""
    mock_url = "http://localhost"
    mock_access_token = "0123456789"

    mock_object = Starling({
        "api_url": mock_url
    })

    with open("test/responses/v1-get-accounts.json") as f:
        expected_response = json.load(f)

        requests_mock.get("{}/api/v1/accounts".format(mock_url),
                          headers=default_headers(mock_access_token),
                          request_headers=expect_authorization_header(mock_access_token),
                          json=expected_response)

        mocked_response = mock_object.get_account(mock_access_token)

        assert expected_response == mocked_response



def default_headers(access_token):
    return {
        "Accept": "application/json",
        "Authorization": "Bearer {access_token}".format(access_token=access_token)
    }


def post_headers(access_token):
    return {
        "Accept": "application/json",
        "Content-Type": "application/json",
        "Authorization": "Bearer {access_token}".format(access_token=access_token)
    }


def oauth_headers():
    return {
        "Content-Type": "application/x-www-form-urlencoded",
        "Accept": "application/json"
    }




import logging

import starling.utils.request as request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

get_contact_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

get_contacts_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

get_contact_account_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "contactId", "validations": ("required", str)}
]

create_contact_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "name", "validations": ("required", str)},
    {"name": "accountNumber", "validations": ("required", str)},
    {"name": "sortCode", "validations": ("required", str)}
]

delete_contact_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "contactId", "validations": ("required", str)}
]


class Contact(object):
    """ Service to interact with a customer's customer's contacts (payees)"""

    def __init__(self, options):
        """
        Creates a new contact service

        :param options: application config
        """
        self.options = options

    def get_contact_account(self, access_token: str, contact_id: str):
        """
        Gets a specific contact from the customer's contacts (payees)

        :param contact_id: the id of the contact
        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], get_contact_parameter_definition)
        url = "{api_url}/api/v1/contacts/{contact_id}".format(api_url=self.options["api_url"], contact_id=contact_id)
        print(url)
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def get_contacts(self, access_token):
        """
        Gets the customer's contacts (payees)

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], get_contacts_parameter_definition)
        url = "{api_url}/api/v1/contacts".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def get_contact_accounts(self, access_token, contact_id):
        """
        Gets a specific contact's (payee) list of accounts

        :param access_token: the oauth bearer token
        :param contact_id: the contact's ID
        :return: the json response dict
        """
        type_validation([access_token, contact_id], get_contact_account_parameter_definition)
        url = "{api_url}/api/v1/contacts/{contact_id}/accounts".format(
            api_url=self.options["api_url"], contact_id=contact_id)
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def create_contact(self, access_token, name, account_number, sort_code):
        """
        Creates a contact (payee) for the customer

        :param access_token: the oauth bearer token
        :param name: the name of the new contact
        :param account_number: the contact's bank account number
        :param sort_code: the contact's sort code
        :return: the json response dict
        """
        type_validation([access_token, name, account_number, sort_code],
                        get_contact_account_parameter_definition)
        url = "{api_url}/api/v1/contacts".format(api_url=self.options["api_url"])
        logging.debug("POST {url}".format(url=url))

        return request.post(
            url,
            headers=default_headers(access_token),
            data={
                "name": name,
                "accountNumber": account_number,
                "sortCode": sort_code
            }
        )

    def delete_contact(self, access_token, contact_id):
        """
        Deletes a specific contact (payee) from the customer's account

        :param access_token: the oauth bearer token
        :param contact_id: the contact's ID
        :return: the json response dict
        """
        type_validation([access_token, contact_id], delete_contact_parameter_definition)
        url = "{api_url}/api/v1/contacts/{contact_id}".format(api_url=self.options["api_url"], contact_id=contact_id)
        logging.debug("DELETE {url}".format(url=url))
        return request.delete(url, headers=default_headers(access_token))



import logging

import starling.utils.request as request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

list_mandates_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

delete_mandate_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "mandateId", "validations": ("required", str)}
]


class Mandate(object):
    """Service to interact with a customer's transactions"""

    def __init__(self, options):
        """
        Create a new transaction service

        :param options: application config
        """
        self.options = options

    def list_mandates(self, access_token):
        """
        Gets a list of the customer's current direct debit mandates

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], list_mandates_parameter_definition)
        url = "{api_url}/api/v1/direct-debit/mandates".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def get_mandate(self, access_token, mandate_id):
        """
        Gets a specific direct debit mandates

        :param access_token: the oauth bearer token
        :param mandate_id: the unique mandate ID
        :return: the json response dict
        """
        type_validation([access_token], list_mandates_parameter_definition)
        url = "{api_url}/api/v1/direct-debit/mandates/{mandate_id}".format(
            api_url=self.options["api_url"], mandate_id=mandate_id)
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def delete_mandate(self, access_token, mandate_id):
        """
        Deletes specific direct debit mandate

        :param access_token: the oauth bearer token
        :param mandate_id: the unique mandate ID
        :return: the json response dict
        """
        type_validation([access_token, mandate_id], delete_mandate_parameter_definition)
        url = "{api_url}/api/v1/direct-debit/mandates/{mandate_id}".format(
            api_url=self.options["api_url"], mandate_id=mandate_id)
        logging.debug("DELETE {url}".format(url=url))
        return request.delete(url, headers=default_headers(access_token))



import logging
from starling.utils.http import default_headers
import starling.utils.request as request
from starling.utils.validator import type_validation, ParameterDefinition

get_account_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

get_balance_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]


class Account(object):
    """ Service to interact with a customer's account """

    def __init__(self, options):
        """
        Creates an instance of the account client

        :param options: application config
        """
        self.options = options

    def get_account(self, access_token):
        """
        Retrieves a customer's account

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], get_account_parameter_definition)
        url = "{api_url}/api/v1/accounts".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def get_balance(self, access_token):
        """
        Retrieves the customer's balance

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], get_balance_parameter_definition)
        url = "{api_url}/api/v1/accounts/balance".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))



import logging

from starling.utils import request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

get_transactions_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "fromDate", "validations": ("optional", str)},
    {"name": "toDate", "validations": ("optional", str)},
    {"name": "source", "validations": ("optional", str)}
]

get_transaction_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "transactionId", "validations": ("required", str)},
    {"name": "source", "validations": ("optional", str)}
]


def transaction_source(source):
    source_map = {
        "MASTER_CARD": "/mastercard",
        "FASTER_PAYMENTS_IN": "/fps/in",
        "FASTER_PAYMENTS_OUT": "/fps/out",
        "DIRECT_DEBIT": "/direct-debit"

    }

    if source in source_map:
        return source_map[source]
    else:
        return ""


class Transaction(object):
    """ Service to interact with a customer's transactions """

    def __init__(self, options):
        """
        Creates an instance of the transaction service

        :param options: application config
        """
        self.options = options

    def get_transactions(self, access_token, from_date, to_date, source):
        """
        Makes a payment on behalf of the customer to another UK bank account using the Faster Payments network

        :param access_token: the oauth bearer token
        :param from_date: filter transactions after this date. Format: YYYY-MM-DD
        :param to_date: filter transactions before this date. Format: YYYY-MM-DD
        :param source: the transaction type (e.g. faster payments, mastercard)
                       if not specified, results are not filtered by source
        :return: the json response dict
        """

        type_validation([access_token, from_date, to_date, source],
                        get_transactions_parameter_definition)

        url = "{api_url}/api/v1/transactions{source}".format(
            api_url=self.options["api_url"], source=transaction_source(source))

        logging.debug("GET {url}".format(url=url))

        return request.get(url, headers=default_headers(access_token), params={
            "from": from_date,
            "to": to_date
        })

    def get_transaction(self, access_token, transaction_id, source):
        """
        Makes a payment on behalf of the customer to another UK bank account using the Faster Payments network

        :param access_token: the oauth bearer token
        :param transaction_id: the unique transaction ID
        :param source: the transaction type (e.g. faster payments, mastercard)
                       if not specified, only generic transaction information is provided
        :return: the json response dict
        """

        type_validation([access_token, transaction_id, source],
                        get_transaction_parameter_definition)

        url = "{api_url}/api/v1/transactions{source}/{id}".format(
            api_url=self.options["api_url"], source=transaction_source(source), id=transaction_id)

        logging.debug("GET {url}".format(url=url))

        return request.get(url, headers=default_headers(access_token))



from starling.entities.account import Account
from starling.entities.address import Address
from starling.entities.card import Card
from starling.entities.contact import Contact
from starling.entities.customer import Customer
from starling.entities.mandate import Mandate
from starling.entities.oauth import OAuth
from starling.entities.payment import Payment
from starling.entities.savingsgoals import SavingsGoals
from starling.entities.transaction import Transaction
from starling.entities.whoami import WhoAmI


class Starling(object):
    """ facade to dispatch operations to services """

    def __init__(self, options):
        """
        create an instance of the starling client
        :param options: configuration parameters
        """

        defaults = {
            "api_url": "https://api.starlingbank.com",
            "oauth_url": "https://oauth.starlingbank.com",
            "client_id": "",
            "client_secret": ""
        }

        self.config = {**defaults, **options}

        self.whoami = WhoAmI(self.config)
        self.customer = Customer(self.config)
        self.account = Account(self.config)
        self.address = Address(self.config)
        self.transaction = Transaction(self.config)
        self.payment = Payment(self.config)
        self.mandate = Mandate(self.config)
        self.contact = Contact(self.config)
        self.card = Card(self.config)
        self.savings_goals = SavingsGoals(self.config)
        self.oauth = OAuth(self.config)

    def get_me(self, access_token=None):
        """
        gets the customer uuid and permissions corresponding to the access token passed

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.whoami.get_me(access_token)

    def get_customer(self, access_token=None):
        """
        gets the customer's details

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.customer.get_customer(access_token)

    def get_account(self, access_token=None):
        """
        gets the customer's account details

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.account.get_account(access_token)

    def get_balance(self, access_token=None):
        """
        gets the customer's balance

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.account.get_balance(access_token)

    def get_addresses(self, access_token=None):
        """
        gets the customer's addresses (current and previous)

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.address.get_addresses(access_token)

    def get_transactions(self, from_date, to_date, source, access_token=None):
        """
        gets the customer's transaction history

        :param from_date: filter transactions after this date. _format: yyyy-mm-dd (optional,
                          defaults to most recent 100 transactions)
        :param to_date: filter transactions before this date. _format: yyyy-mm-dd (optional,
                        defaults to current date if not provided)
        :param source: the transaction type (e.g. faster payments, mastercard).
                       if not specified, results are not filtered by source.
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.transaction.get_transactions(access_token, from_date, to_date, source)

    def get_transaction(self, transaction_id, source, access_token=None):
        """
        gets the full details of a single transaction

        :param transaction_id: the unique transaction id
        :param source: the transaction type (e.g. faster payments, mastercard).
                       if not specified, only generic transaction information will be returned.
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.transaction.get_transaction(access_token, transaction_id, source)

    def list_mandates(self, access_token=None):
        """
        gets the customer's current direct-debit mandates

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.mandate.list_mandates(access_token)

    def get_mandate(self, mandate_id, access_token=None):
        """
        gets a specific direct-debit mandate

        :param mandate_id: the unique mandate id
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.mandate.get_mandate(access_token, mandate_id)

    def delete_mandate(self, mandate_id, access_token=None):
        """
        gets the customer's current direct-debit mandates

        :param mandate_id: the unique mandate id
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.mandate.delete_mandate(access_token, mandate_id)

    def list_scheduled_payments(self, access_token=None):
        """
        lists the customer's scheduled payments

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.payment.list_scheduled_payments(access_token)

    def make_local_payment(self, destination_account_uid, reference, amount, access_token=None):
        """
        makes a payment on behalf of the customer to another uk bank account using the _faster _payments network

        :param destination_account_uid: the account identifier of the recipient
        :param reference: the payment reference, max. 18 characters
        :param amount: the amount to be send
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.payment.make_local_payment(access_token, destination_account_uid, reference, amount)

    def get_contacts(self, access_token=None):
        """
        gets the customer's contacts (payees)

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.contact.get_contacts(access_token)

    def get_contact_account(self, contact_id, access_token=None):
        """
        gets a specific contact (payee)

        :param contact_id: the contact's id
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.contact.get_contact_account(access_token, contact_id)

    def create_contact(self, name, account_number, sort_code, access_token=None):
        """
        creates a contact (payee) for the customer

        :param name: the name of the new contact
        :param account_number: the contact's bank account number
        :param sort_code: the contact's sort code
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used.
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.contact.create_contact(access_token, name, account_number, sort_code)

    def delete_contact(self, contact_id, access_token=None):
        """
        deletes a contact (payee) for the customer

        :param contact_id: the contact's id
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.contact.delete_contact(access_token, contact_id)

    def list_savings_goals(self, access_token=None):
        """
        gets a list of the customer's savings goals

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.savings_goals.list_savings_goals(access_token)

    def get_savings_goal(self, savings_goal_id, access_token=None):
        """
        gets a specific savings goal

        :param savings_goal_id:
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.savings_goals.get_savings_goal(access_token, savings_goal_id)

    def add_money_to_savings_goal(self, savings_goal_id, transaction_id, amount, currency="GBP", access_token=None):
        """
        add money to a specific savings goal

        :param savings_goal_id: the savings goal's id
        :param transaction_id: a transaction id for this transaction
        :param amount: an amount in minor unit
        :param currency: the currency unit
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.savings_goals.add_money_to_savings_goal(
            access_token,
            savings_goal_id,
            transaction_id,
            amount,
            currency
        )

    def create_savings_goal(self, savings_goal_id, name, target_amount, currency="GBP",
                            target_currency="GBP", base64_encoded_photo=None, access_token=None):
        """
        create a new savings goal

        :param savings_goal_id: the savings goal's id, generate one if creating a goal
        :param name: the name of the new contact
        :param target_amount: the target amount in minor units (e.g. 1234 => £12.34)
        :param currency: the currency of the savings goal. defaults to "GBP"
        :param target_currency: the target currency, also defaults to "GBP"
        :param base64_encoded_photo: base64 encoded image to associate with the goal. (optional)
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        # if access_token is None:
        #     access_token = self.config["access_token"]
        #
        #     f, access_token, savings_goal_id, name, currency,
        #     target_amount, target_currency = "GBP", base64_encoded_photo = None):

        return self.savings_goals.create_savings_goal(access_token, savings_goal_id, name, target_amount,
                                                      currency=currency, target_currency=target_currency,
                                                      base64_encoded_photo=base64_encoded_photo)

    def delete_savings_goal(self, savings_goal_id, access_token=None):
        """
        deletes specific direct debit mandate

        :param savings_goal_id: the savings goal's id
        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """
        if access_token is None:
            access_token = self.config["access_token"]

        return self.savings_goals.delete_savings_goal(access_token, savings_goal_id)

    def get_card(self, access_token=None):
        """
        gets the customer's card

        :param access_token: the oauth bearer token. if not specified, the access_token on the options object is used
        :return: the json response dict
        """

        if access_token is None:
            access_token = self.config["access_token"]

        return self.card.get_card(access_token)

    def get_access_token(self, authorization_code):
        """
        exchanges the authorization code for an access token

        :param self:
        :param authorization_code: the authorization code, acquired from the user agent after the
                                  user authenticates with starling
        :return: the json response dict
        """

        return self.oauth.get_access_token(authorization_code)

    def refresh_access_token(self, refresh_token):
        """
        exchanges the authorization code for an access token

        :param refresh_token: the oauth refresh token, used to claim a new access token when the access token
                             expires. a new refresh token is also returned
        :return:
        """
        return self.oauth.refresh_access_token(refresh_token)



import setuptools
from setuptools import setup

setup(name="pystarling",
      version="0.0.5",
      description="A python SDK for interacting with the Starling V1 API.",
      url="https://github.com/aranscope/starling-python-sdk",
      author="Aran Long",
      author_email="me@aran.site",
      license="MIT",
      packages=setuptools.find_packages(exclude=["test"]),
      install_requires=["requests"],
      )



from collections import namedtuple

from starling.errors import TypeValidationError

ParameterDefinition = namedtuple("Validation", ("name", "definitions"))


def _run_validations(pos, name, validations, value=None):

    if not validations:
        return None

    if validations[0] == "optional":
        if value is not None and not isinstance(value, validations[1]):
            return "{name} parameter in position {pos} is an optional {rule} but was {value_type}".format(
                name=name,
                pos=pos,
                rule=validations[1],
                value_type=type(value)
            )

    if validations[0] == "required":
        if value is not None and not isinstance(value, validations[1]):
            return "{name} parameter in position {pos} is an optional {rule} but was {value_type}".format(
                name=name,
                pos=pos,
                rule=validations[1],
                value_type=type(value)
            )
        elif value is None:
            return "{name} parameter in position {pos} is an optional {rule} but was {value}".format(
                name=name,
                pos=pos,
                rule=validations[1],
                value=value
            )

    return None


def type_validation(arguments, definitions):
    problems = []

    for pos, definition in enumerate(definitions):

        if arguments:
            problem = _run_validations(pos, definition["name"], definition["validations"], arguments[pos])
        else:
            problem = _run_validations(pos, definition["name"], definition["validations"])

        if problem is not None:
            problems.append(problem)

    if problems:
        raise TypeValidationError(problems)




import logging

import starling.utils.request as request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

get_customer_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]


class Customer(object):
    """Service to interact with a customer"""

    def __init__(self, options):
        """
        Creates a instance of the customer client

        :param options: application config
        """
        self.options = options

    def get_customer(self, access_token):
        """
        Gets a customer's details

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], get_customer_parameter_definition)
        url = "{api_url}/api/v1/customers".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))



import logging

import starling.utils.request as request
from starling.utils.http import default_headers
from starling.utils.validator import type_validation

list_savings_goals_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)}
]

get_savings_goals_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "savingsGoalId", "validations": ("required", str)}
]

delete_savings_goals_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "savingsGoalId", "validations": ("required", str)}
]

create_savings_goals_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "name", "validations": ("required", str)},
    {"name": "currency", "validations": ("required", str)},
    {"name": "targetAmount", "validations": ("optional", str)},
    {"name": "base64EncodedPhoto", "validations": ("optional", str)}
]

add_money_to_savings_goal_parameter_definition = [
    {"name": "accessToken", "validations": ("required", str)},
    {"name": "savingsGoalId", "validations": ("required", str)},
    {"name": "transactionId", "validations": ("required", str)},
    {"name": "amount", "validations": ("required", int)}
]


class SavingsGoals(object):
    """ Service to interact with a customer's savings goals """

    def __init__(self, options):
        """
        Creates an instance of the savings goals service

        :param options: application config
        """
        self.options = options

    def list_savings_goals(self, access_token):
        """
        Gets a lists of the customer's savings goals

        :param access_token: the oauth bearer token
        :return: the json response dict
        """
        type_validation([access_token], list_savings_goals_parameter_definition)
        url = "{api_url}/api/v1/savings-goals".format(api_url=self.options["api_url"])
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def get_savings_goal(self, access_token, savings_goal_id):
        """
        Gets a specific savings goal

        :param access_token: the oauth bearer token
        :param savings_goal_id: the savings goal's ID
        :return: the json response dict
        """
        type_validation([access_token, savings_goal_id], get_savings_goals_parameter_definition)
        url = "{api_url}/api/v1/savings-goals/{goal_id}".format(
            api_url=self.options["api_url"], goal_id=savings_goal_id)
        logging.debug("GET {url}".format(url=url))
        return request.get(url, headers=default_headers(access_token))

    def create_savings_goal(self, access_token, savings_goal_id, name,
                            target_amount, currency="GBP", target_currency="GBP", base64_encoded_photo=None):
        """
        Creates a savings goal

        :param access_token: the oauth bearer token
        :param savings_goal_id: the saving's goal's ID
        :param name: the name of the savings goal
        :param target_amount: the target amount in minor units (e.g. 1234 => £12.34)
        :param currency: the currency of the savings goal, defaults to 'GBP'
        :param target_currency: the target currency, defaults to 'GBP'
        :param base64_encoded_photo: base64 encoded image to associate with the goal. (optional)
        :return: the json response dict
        """

        type_validation([access_token, name, currency,
                         target_amount, target_currency, base64_encoded_photo],
                        create_savings_goals_parameter_definition)

        data = {
            "name": name,
            "currency": currency,
            "target": {
                "targetAmount": target_amount,
                "targetCurrency": target_currency
            }
        }

        if base64_encoded_photo is not None:
            data["base64EncodedPhoto"] = base64_encoded_photo

        url = "{api_url}/api/v1/savings-goals/{goal_id}".format(
            api_url=self.options["api_url"], goal_id=savings_goal_id)

        logging.debug("PUT {url}".format(url=url))

        return request.put(url, headers=default_headers(access_token), data=data)

    def delete_savings_goal(self, access_token, savings_goal_id):
        """
        Deletes a specific savings goal

        :param access_token: the oauth bearer token
        :param savings_goal_id: the savings goal's ID
        :return: the json response dict
        """
        type_validation([access_token, savings_goal_id], delete_savings_goals_parameter_definition)
        url = "{api_url}/api/v1/savings-goals/{goal_id}".format(
            api_url=self.options["api_url"], goal_id=savings_goal_id)
        logging.debug("DELETE {url}".format(url=url))
        return request.delete(url, headers=default_headers(access_token))

    def add_money_to_savings_goal(self, access_token, savings_goal_id, transaction_id, amount, currency):
        """
        Add money to a specific savings goal

        :param access_token: the oauth bearer token
        :param savings_goal_id: the saving's goal's ID
        :param transaction_id:
        :param amount: an amount in minor unit
        :param currency: the currency of the savings goal

        :return: the json response dict
        """

        type_validation([access_token, savings_goal_id, transaction_id, amount, currency],
                        add_money_to_savings_goal_parameter_definition)

        url = "{api_url}/api/v1/savings-goals/{goal_id}/add-money/{transaction_id}".format(
            api_url=self.options["api_url"], goal_id=savings_goal_id, transaction_id=transaction_id)

        logging.debug("PUT {url}".format(url=url))

        return request.put(url, headers=default_headers(access_token), data={
            "amount": {
                "currency": currency,
                "targetCurrency": currency
            }
        })



"""
Errors defined in Starling docs Errors section.
https://developer.starlingbank.com/docs
"""


class BadRequestError(Exception):
    """An exception raised when something is wrong with the request made"""


class UnauthorizedError(Exception):
    """An exception raised when you are not authorized to access the requested data"""


class ForbiddenError(Exception):
    """An exception raised when authentication is failed, usually due to an expired token,
       or attempting to access a resource beyond the scope of the token"""


class NotFoundError(Exception):
    """An exception raised when the requested resource does not exist"""


class InternalServerError(Exception):
    """An exception raised when Starling has an internal server error"""


class TypeValidationError(Exception):
    """An exception raised when a type validation fails"""

    def __init__(self, validation_errors):
        self.validation_errors = validation_errors


class ClientConfigurationError(Exception):
    """Raised when a client is misconfigured"""


class UnhandledStatusCodeError(Exception):
    """Raised when an unexpected status code is received"""



import json

from starling.starling import Starling
from starling.utils.http import oauth_headers
import os

def test_get_access_token(requests_mock):
    mock_object = Starling({
        "oauth_url": "http://localhost",
        "client_id": "myclientid",
        "client_secret": "myclientsecret",
        "redirect_uri": "redirect"
    })

    print(os.getcwd())

    with open("test/responses/v1-get-access-token.json") as f:
        expected_response = json.load(f)

        requests_mock.post("http://localhost/oauth/access-token",
                           request_headers=oauth_headers(),
                           json=expected_response)

        mock_response = mock_object.get_access_token("code")

        assert expected_response == mock_response


def test_refresh_access_token(requests_mock):
    mock_object = Starling({
        "oauth_url": "http://localhost",
        "client_id": "myclientid",
        "client_secret": "myclientsecret",
        "redirect_uri": "redirect"
    })

    with open("test/responses/v1-refresh-access-token.json") as f:
        expected_response = json.load(f)

        requests_mock.post("http://localhost/oauth/access-token",
                           request_headers=oauth_headers(),
                           json=expected_response)

        mock_response = mock_object.refresh_access_token(
            "11tyjapK8Vx3mBbCMkCrTmlbxjVRSny16MmbvGSvRlwhKYmFC4dbZetV0nTcXGjT")

        assert expected_response == mock_response
